import serial
from clunet_client import ClunetPacket

class ClunetGpioDriver():
    def __init__(self, device_path):
        self.clunet_bus_reader = open(device_path, "r")
        print("Opened {} for reading".format(device_path))
        self.clunet_bus_writer = open(device_path, "w")
        print("Opened {} for writing".format(device_path))
    
    @staticmethod
    def check_crc(data):
        crc = 0
        for i in range(len(data)):
            inbyte = data[i]
            for j in range(8):
                mix = (crc ^ inbyte) & 1
                crc = crc >> 1
                if mix != 0:
                    crc = crc ^ 0x8C
                inbyte = inbyte >> 1
        return crc

    def recv(self):
        line = self.clunet_bus_reader.readline().strip(" \r\n")
        if len(line) == 0:
            self.recv()
        return ClunetPacket.parse(line)
    
    def send(self, packet):
        print(packet.encode(), file=self.clunet_bus_writer, flush=True)

    def close(self):
        if self.clunet_bus_reader != None:
            try:
                self.clunet_bus_reader.close()
            except:
                pass
            self.clunet_bus_reader = None
        if self.clunet_bus_writer != None:
            try:
                self.clunet_bus_writer.close()
            except:
                pass
            self.clunet_bus_writer = None

    def __enter__(self):
        return self
    
    def __exit__(self, exp_type, exp_value, exp_tr):
        self.close()
