import serial
from clunet_client import ClunetPacket

class ClunetSerial():
    def __init__(self, port, speed):
        self._clunet_ser = serial.Serial(port, speed)
        print("Opened {} for reading".format(port))
    
    @staticmethod
    def check_crc(data):
        crc = 0
        for i in range(len(data)):
            inbyte = data[i]
            for j in range(8):
                mix = (crc ^ inbyte) & 1
                crc = crc >> 1
                if mix != 0:
                    crc = crc ^ 0x8C
                inbyte = inbyte >> 1
        return crc

    def recv(self):
        line = self._clunet_ser.readline()
        line = line.decode("ascii")
        line = line.strip(" \r\n")
        if len(line) == 0 or line == "!":
            return self.recv()
        if line[0] == "!":
            print("Warning:", line)
            return self.recv()

        buffer = bytes.fromhex(line)
        if ClunetSerial.check_crc(buffer) != 0:
            raise IOError("Bad CRC for '{}'".format(line))
        prio = 0 # priority is unknown
        src = buffer[0]
        dst = buffer[1]
        cmd = buffer[2]
        l = buffer[3]
        data = buffer[4:4+l]

        packet = ClunetPacket(prio=prio, src=src, dst=dst, cmd=cmd, data=data)
        return packet
    
    def send(self, packet):
        buffer = []
        buffer.append(packet.prio)
        buffer.append(packet.dst)
        buffer.append(packet.cmd)
        buffer.extend(list(packet.data))
        buffer.append(ClunetSerial.check_crc(buffer))
        line = bytes(buffer).hex().upper()+"\r\n"
        self._clunet_ser.write(line.encode('ascii'))

    def close(self):
        if self._clunet_ser != None:            
            try:
                self._clunet_ser.close()
            except:
                pass
        self._clunet_ser = None

    def __enter__(self):
        return self
    
    def __exit__(self, exp_type, exp_value, exp_tr):
        self.close()
